//This file is part of the LArray v1.0 library written in -*- C++ -*-

//LArray v1.0: a simple C++ array class
//Copyright (C) Luca Saiu, 1999-2000

//This is free software; you can redistribute it and/or modify it
//under the terms of the GNU Lesser General Public License version 2.1
//as published by the Free Software Foundation.

//This software is distributed in the hope that it will be useful, but
//WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public
//License along with this file; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.


//Luca Saiu
//------------------------------------------
//e-mail:      luca78@dada.it
//paper mail:  Via Camporcioni, 15
//             51019 Ponte Buggianese - Pistoia
//             Italy
//telephone:   0572-635477 Italy


#ifndef LARRAY_H_
#define LARRAY_H_

// Comment out next line if you don't want bound checks
#define LARRAY_BOUND_CHECKING

#include "Lgen/Lgen.h"
#include <iostream.h>

extern "C"{
#include <stdlib.h>
}

class LOutOfBoundsException:LException{};

template <class T> class LArray{
protected:
  unsigned long nelements; // Number of elements
  T** buffer;              // Buffer of pointers to T's
public:
  // Constructors and destructor
  LArray();
  LArray(const LArray<T>&); // Copy constructor
  ~LArray();

  // Methods
  void SetDim(unsigned long);
  unsigned long GetDim() const;
  void Append(const T x);
  void Scramble();
  int Search(const T x) const; // Returns the index i of the first element
                               // such that (*this)[i]==x, or -1 if no
                               // matching elements exist.
  bool Has(const T x) const;

  // Overloaded operators
  T& operator[](unsigned long);
  const T& operator[](unsigned long) const;
  bool operator==(const LArray<T>) const;  // Two arrays are equal iff all
                                           // their elements are equal
  bool operator!=(const LArray<T>) const;
  const LArray<T>& operator=(const LArray<T>&);   // Assignment operator
  const LArray<T> operator+(const LArray<T> x) const;
  const LArray<T>& operator+=(const LArray<T> x);

  void Sort();

  // stream functions
  friend ostream& operator<< <>(ostream &s,const LArray<T> &a);
};

// Implementation
////////////////////////////////////////////////////////////////////
extern "C"{
#include <malloc.h>
}

template <class T> LArray<T>::LArray():buffer(0),
                                       nelements(0){ // Buffer is empty
  // Do nothing more than initialization
}

template <class T> LArray<T>::LArray(const LArray<T> &A):buffer(0),
                                                         nelements(0){
  // After initialization, just copy A into *this
  operator=(A);
}

template <class T> LArray<T>::~LArray(){
  unsigned long i; // Loop counter

  // Destroy every element
  for(i=0;i<nelements;i++)
     delete buffer[i];

  // Free buffer storage
  free(buffer);
}

template <class T> bool LArray<T>::operator==(const LArray<T> A) const{
  int i; // Loop counter

  // If there is even only a different element, then return false...
  for(i=0;i<nelements;i++)
    if( (*(buffer[i])) != (*(A.buffer[i])) )
      return false;

  // ...but if we arrive here the arrays are equal
  return true;
}

template <class T> bool LArray<T>::operator!=(const LArray<T> A) const{
  return !operator==(A);
}

template <class T> const LArray<T>& LArray<T>::operator=(const LArray<T> &A){
  // Set dimension:
  SetDim(A.GetDim());

  // Copy elements:
  unsigned long i;
  for(i=0;i<GetDim();i++)
    operator[](i)=A[i];

  return *this;
}

template <class T> void LArray<T>::SetDim(unsigned long newdim){
  // If the new dimension is the same as the current dimension do nothing
  if(newdim==nelements)
    return;

  unsigned long i; // Loop counter

  // If newdim is greater than nelements...
  if(newdim>nelements){
    // ...then resize the buffer...
    if(nelements!=0)    // buffer was allocated before is not 0, I can use realloc()
      buffer=(T**)xrealloc(buffer,sizeof(T*)*newdim);
    else             // buffer was never allocated before, can't use realloc()
      buffer=(T**)xmalloc(sizeof(T*)*newdim);

    // ...and create the new elements
    for(i=nelements;i<newdim;i++)
      buffer[i]=new T;
  }
  else{ // If newdim is less than neelements...
    // ...then destroy every element in the interval [newdim,nelements)
    for(i=newdim;i<nelements;i++)
      delete buffer[i];

    // ...and resize the buffer
    if(newdim!=0)
      buffer=(T**)xrealloc(buffer,sizeof(T*)*newdim);
    else
      free(buffer);
  }

  // Update nelements
  nelements=newdim;
}

template <class T> unsigned long LArray<T>::GetDim() const{
  // Return the number of elements
  return nelements;
}

template <class T> T& LArray<T>::operator[](unsigned long n){
  // If the LARRAY_BOUND_CHECKING macro is defined, then check bounds
  #ifdef LARRAY_BOUND_CHECKING
  if((n>=nelements)||(n<0))
    throw LOutOfBoundsException();
  #endif

  // Return a reference to the object pointed by the n-th element of
  // buffer
  return *(buffer[n]);
}

template <class T> const T& LArray<T>::operator[](unsigned long n) const{
  // If the LARRAY_BOUND_CHECKING macro is defined, then check bounds
  #ifdef LARRAY_BOUND_CHECKING
  if((n>=nelements)||(n<0))
    throw LOutOfBoundsException();
  #endif

  // Return a reference to the object pointed by the n-th element of
  // buffer
  return *(buffer[n]);
}

template <class T> ostream& operator<<(ostream &s,const LArray<T> &a){
  unsigned long i; // Loop counter

  // Write "#("
  s<<"#(";

  // Write every element followed by a space, except the last one,
  // or nothing if the array is empty
  for(i=0;i+1<a.nelements;i++) // I cannot say "i<a.nelements-1",
                               // because a.nelements is unsigned
    s<<*(a.buffer[i])<<' ';

  // Write the last element followed by a ")", or simply ")" if the
  // array is empty
  if(a.nelements==0)
    s<<")";
  else
    s<<(*(a.buffer[a.nelements-1]))<<')';

  return s;
}

template <class T> void LArray<T>::Append(const T x){
  // Make place for a new cell at the end of *this and write x it in
  // the last cell:
  SetDim(GetDim()+1);
  operator[](GetDim()-1)=x;
}

template <class T> bool LArray<T>::Has(const T x) const{
  // Return true iff search succeeds:
  return Search(x)!=-1;
}

template <class T> int LArray<T>::Search(const T x) const{
  // Scan the array looking for x:
  int i;
  for(i=0;i<GetDim();i++)
    if(operator[](i)==x)
      return i;  // Ok, we found x at the i-th position:

  // We did not find x; return -1 to indicate failure:
  return -1;
}

template <class T> const LArray<T> LArray<T>::operator+(const LArray<T> x) const{
  LArray<T> r(*this); // The array to return;

  // Append every element of x to r, then return r:

  // Resize r:
  unsigned long olddim=r.GetDim();
  r.SetDim(olddim+x.GetDim());

  // Copy each element of x into the last part of r:
  int i;
  for(i=0;i<x.GetDim();i++)
    r[olddim+i]=x[i];

  return r;
}

template <class T> const LArray<T>& LArray<T>::operator+=(const LArray<T> x){
  // Append x to yourself, and assign the value to yourself:
  *this=(*this)+x;

  // Return a constant reference to the object itself:
  return *this;
}

#define RANDOM(n) long(float(rand())*n/RAND_MAX)

template <class T> void LArray<T>::Scramble(){ // To do: test for bugs,
                                               // it seems not to work at
                                               // the first call. It works
                                               // ok from the second on.
  int ct,temp;
  T* swap;
  for(ct=0;ct<GetDim();ct++){
    swap=buffer[ct];
    temp=RANDOM(GetDim());
    buffer[ct]=buffer[temp];
    buffer[temp]=swap;
  }
}

#undef RANDOM

template <class T> int _LArrayElementsCompareFunction(const void* a,
                                                      const void* b){
  const T** Ta=(const T**)a;
  const T** Tb=(const T**)b;

  if(**Ta == **Tb)
    return 0;
  else if(**Ta <= **Tb)
    return -1;
  else
    return +1;
}

template <class T> void LArray<T>::Sort(){
  qsort(buffer,(size_t)GetDim(),sizeof(T*),_LArrayElementsCompareFunction<T>);
}

#endif // #ifndef LARRAY_H_

