//This file is part of the LTakeshi v0.0 library written in -*- C++ -*-

//LTakeshi v0.0: a multi-platform C++ library for GUI management
//Copyright (C) Luca Saiu, 1999-2000

//This is free software; you can redistribute it and/or modify it
//under the terms of the GNU Lesser General Public License version 2.1
//as published by the Free Software Foundation.

//This software is distributed in the hope that it will be useful, but
//WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public
//License along with this file; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.


//Luca Saiu
//------------------------------------------
//e-mail:      positrone@freemail.it
//paper mail:  Via Camporcioni, 15
//             51019 Ponte Buggianese - Pistoia
//             Italy
//telephone:   0572-635477 Italy


#include "LAttributeTable.h"
#include "../Lgen/Lgen.h"

// Hash function for attribute table
INDEX hashfunction(const LName n){
  return n;
}

// Content of HashFlags hash table
typedef unsigned char flags;
#define LFLAG_MODIFIED   1  // This attribute was changed and the state
                            // change is still pending

#define LFLAG_READONLY   2  // This attribute can be queried but cannot
                            // be modified

#define LFLAGS_DEFAULT   LFLAG_MODIFIED
                            // Default values of flags

LAttributeTable::LAttributeTable():HashInt(hashfunction),
                                   HashDouble(hashfunction),
                                   HashBool(hashfunction),
                                   HashString(hashfunction),
                                   HashPointer(hashfunction),
                                   HashCallBack(hashfunction),
                                   HashFlags(hashfunction){
#ifdef DEBUG
  cerr<<"An LAttributeTable object has been created.\n";
#endif
}

LAttributeTable::~LAttributeTable(){
#ifdef DEBUG
  cerr<<"An LAttributeTable object has been destroyed.\n";
#endif
}

int LAttributeTable::GetInt(const LName n){
  if(!HashInt.exists(n))
    fatal("LAttributeTable::GetInt(): attribute does not exist");
  return HashInt[n];
}

double LAttributeTable::GetDouble(const LName n){
  if(!HashDouble.exists(n))
    fatal("LAttributeTable::GetDouble(): attribute does not exist");
  return HashDouble[n];
}

bool LAttributeTable::GetBool(const LName n){
  if(!HashBool.exists(n))
    fatal("LAttributeTable::GetBool(): attribute does not exist");
  return HashBool[n];
}

Lstring LAttributeTable::GetString(const LName n){
  if(!HashString.exists(n))
    fatal("LAttributeTable::GetString(): attribute does not exist");
  return HashString[n];
}

void* LAttributeTable::GetPointer(const LName n){
  if(!HashPointer.exists(n))
    fatal("LAttributeTable::GetPointer(): attribute does not exist");
  return HashPointer[n];
}

LCallBack LAttributeTable::GetCallBack(const LName n){
  if(!HashCallBack.exists(n))
    fatal("LAttributeTable::GetCallBack(): attribute does not exist");
  return HashCallBack[n];
}

void LAttributeTable::SetInt(const LName n,int x){
  if(!HashInt.exists(n))
    fatal("LAttributeTable::SetInt(): attribute does not exist");
  if(HashFlags[n]&LFLAG_READONLY)
    fatal("LAttributeTable::SetInt(): tried to set a read-only attribute");
  HashFlags[n]|=LFLAG_MODIFIED;
  HashInt[n]=x;
  return;
}

void LAttributeTable::SetDouble(const LName n,double x){
  if(!HashDouble.exists(n))
    fatal("LAttributeTable::SetDouble(): attribute does not exist");
  if(HashFlags[n]&LFLAG_READONLY)
    fatal("LAttributeTable::SetDouble(): tried to set a read-only attribute");
  HashFlags[n]|=LFLAG_MODIFIED;
  HashDouble[n]=x;
  return;
}

void LAttributeTable::SetBool(const LName n,bool x){
  if(!HashBool.exists(n))
    fatal("LAttributeTable::SetBool(): attribute does not exist");
  if(HashFlags[n]&LFLAG_READONLY)
    fatal("LAttributeTable::SetBool(): tried to set a read-only attribute");
  HashFlags[n]|=LFLAG_MODIFIED;
  HashBool[n]=x;
  return;
}

void LAttributeTable::SetString(const LName n,Lstring x){
  if(!HashString.exists(n))
    fatal("LAttributeTable::SetString(): attribute does not exist");
  if(HashFlags[n]&LFLAG_READONLY)
    fatal("LAttributeTable::SetString(): tried to set a read-only attribute");
  HashFlags[n]|=LFLAG_MODIFIED;
  HashString[n]=x;
  return;
}

void LAttributeTable::SetPointer(const LName n,void *x){
  if(!HashPointer.exists(n))
    fatal("LAttributeTable::SetPointer(): attribute does not exist");
  if(HashFlags[n]&LFLAG_READONLY)
    fatal("LAttributeTable::SetPointer(): tried to set a read-only attribute");
  HashFlags[n]|=LFLAG_MODIFIED;
  HashPointer[n]=x;
  return;
}

void LAttributeTable::SetCallBack(const LName n,LCallBack x){
  if(!HashCallBack.exists(n))
    fatal("LAttributeTable::SetCallBack(): attribute does not exist");
  if(HashFlags[n]&LFLAG_READONLY)
    fatal("LAttributeTable::SetCallBack(): tried to set a read-only attribute");
  HashFlags[n]|=LFLAG_MODIFIED;
  HashCallBack[n]=x;
  return;
}

void LAttributeTable::InsertInt(const LName n,bool readonly,const int x){
  if(HashInt.insert(n,x)!=EXIT_SUCCESS){
    cerr<<"LAttributeTable: an int attribute has been inserted more "
          "than once\n";
  }
  HashFlags.insert(n,LFLAGS_DEFAULT|(readonly?LFLAG_READONLY:0));
  return;
}

void LAttributeTable::InsertDouble(const LName n,bool readonly,const double x){
  if(HashDouble.insert(n,x)!=EXIT_SUCCESS){
    cerr<<"LAttributeTable: a double attribute has been inserted more "
          "than once\n";
  }
  HashFlags.insert(n,LFLAGS_DEFAULT|(readonly?LFLAG_READONLY:0));
  return;
}

void LAttributeTable::InsertBool(const LName n,bool readonly,const bool x){
  if(HashBool.insert(n,x)!=EXIT_SUCCESS){
    cerr<<"LAttributeTable: a bool attribute has been inserted more "
          "than once\n";
  }
  HashFlags.insert(n,LFLAGS_DEFAULT|(readonly?LFLAG_READONLY:0));
  return;
}

void LAttributeTable::InsertString(const LName n,bool readonly,const Lstring x){
  if(HashString.insert(n,x)!=EXIT_SUCCESS){
    cerr<<"LAttributeTable: a Lstring attribute has been inserted more "
          "than once\n";
  }
  HashFlags.insert(n,LFLAGS_DEFAULT|(readonly?LFLAG_READONLY:0));
  return;
}

void LAttributeTable::InsertPointer(const LName n,bool readonly,void *x){
  if(HashPointer.insert(n,x)!=EXIT_SUCCESS){
    cerr<<"LAttributeTable: a void* attribute has been inserted more "
          "than once\n";
  }
  HashFlags.insert(n,LFLAGS_DEFAULT|(readonly?LFLAG_READONLY:0));
  return;
}

void LAttributeTable::InsertCallBack(const LName n,bool readonly,const LCallBack x){
  if(HashCallBack.insert(n,x)!=EXIT_SUCCESS){
    cerr<<"LAttributeTable: an LCallBack attribute has been inserted more "
          "than once\n";
  }
  HashFlags.insert(n,LFLAGS_DEFAULT|(readonly?LFLAG_READONLY:0));
  return;
}

bool LAttributeTable::Exists(const LName n){
  return HashFlags.exists(n); // It is enough to control the field table,
                              // because every attribute has a flag bit field
}

bool LAttributeTable::WasModified(const LName n){
  return HashFlags[n]&LFLAG_MODIFIED;
}

void LAttributeTable::Reset(const LName n){
  HashFlags[n]&= ~LFLAG_MODIFIED;
  return;
}

bool LAttributeTable::TestAndSet(const LName n){
  bool r=HashFlags[n]&LFLAG_MODIFIED;

  HashFlags[n]&= ~LFLAG_MODIFIED;
  return r;
}

void LAttributeTable::MakeReadOnly(const LName n){
  HashFlags[n]|=LFLAG_READONLY;
  return;
}

void LAttributeTable::Dump(){
  cerr<<"Not yet implemented\n";
}













